@@ -14,20 +14,25 @@ from yellowdog_client.common.json import Json
 from yellowdog_client.model import (
     BestComputeSourceReport,
     BestComputeSourceReportSource,
+    ComputeRequirement,
     ComputeRequirementDynamicTemplateTestResult,
-    ComputeRequirementSummary,
     ComputeRequirementTemplateTestResult,
     ComputeRequirementTemplateUsage,
     ConfiguredWorkerPool,
+    NodeStatus,
+    NodeSummary,
+    ObjectDetail,
     ObjectPath,
     ProvisionedWorkerPool,
     ProvisionedWorkerPoolProperties,
     Task,
     TaskGroup,
+    WorkerPool,
     WorkerPoolSummary,
     WorkRequirement,
     WorkRequirementSummary,
 )
+from yellowdog_client.object_store.upload import UploadBatchBuilder
 
 from yd_commands.args import ARGS_PARSER
 from yd_commands.compact_json import CompactJSONEncoder
@@ -80,7 +85,7 @@ TYPE_MAP = {
     ConfiguredWorkerPool: "Configured Worker Pool",
     ProvisionedWorkerPool: "Provisioned Worker Pool",
     WorkerPoolSummary: "Worker Pool",
-    ComputeRequirementSummary: "Compute Requirement",
+    ComputeRequirement: "Compute Requirement",
     Task: "Task",
     TaskGroup: "Task Group",
     WorkRequirementSummary: "Work Requirement",
@@ -96,19 +101,34 @@ def get_type_name(obj: Item) -> str:
 
 
 def compute_requirement_table(
-    cr_summary_list: List[ComputeRequirementSummary],
-) -> List[List]:
+    cr_list: List[ComputeRequirement],
+) -> (List[str], List[List]):
+    headers = [
+        "#",
+        "Compute Requirement Name",
+        "Tag",
+        "Instance Counts",
+        "Status",
+    ]
     table = []
-    for index, cr_summary in enumerate(cr_summary_list):
+    for index, cr in enumerate(cr_list):
         table.append(
             [
                 index + 1,
-                ":",
-                cr_summary.name,
-                f"[{cr_summary.status}]",
+                cr.name,
+                f"{cr.tag}",
+                (
+                    f"Target={cr.targetInstanceCount:,d},"
+                    f" Expected={cr.expectedInstanceCount:,d}"
+                ),
+                (
+                    f"{cr.status}"
+                    if cr.nextStatus is None
+                    else f"{cr.status} -> {cr.nextStatus}"
+                ),
             ]
         )
-    return table
+    return headers, table
 
 
 def work_requirement_table(
@@ -135,7 +155,6 @@ def task_group_table(
         table.append(
             [
                 index + 1,
-                ":",
                 task_group.name,
                 f"[{task_group.status}]",
             ]
@@ -157,19 +176,57 @@ def task_table(task_list: List[Task]) -> List[List]:
     return table
 
 
-def worker_pool_table(worker_pool_summaries: List[WorkerPoolSummary]) -> List[List]:
+def worker_pool_table(
+    client: PlatformClient, worker_pool_summaries: List[WorkerPoolSummary]
+) -> (List[str], List[List]):
+    headers = [
+        "#",
+        "Provisioned Worker Pool Name",
+        "Status",
+        "Nodes: Running",
+        "Min",
+        "Max",
+        "Terminated",
+        "Late",
+        "Lost",
+        "Deregistered",
+    ]
     table = []
     for index, worker_pool_summary in enumerate(worker_pool_summaries):
+        worker_pool: WorkerPool = client.worker_pool_client.get_worker_pool_by_id(
+            worker_pool_summary.id
+        )
+        try:
+            min_nodes = str(worker_pool.properties.minNodes)
+        except:
+            min_nodes = " "
+        try:
+            max_nodes = str(worker_pool.properties.maxNodes)
+        except:
+            max_nodes = " "
+        node_summary: NodeSummary = worker_pool.nodeSummary
+        nodes_running = node_summary.statusCounts[NodeStatus.RUNNING]
+        nodes_late = node_summary.statusCounts[NodeStatus.LATE]
+        nodes_lost = node_summary.statusCounts[NodeStatus.LOST]
+        nodes_terminated = node_summary.statusCounts[NodeStatus.TERMINATED]
+        nodes_deregistered = node_summary.statusCounts[NodeStatus.DEREGISTERED]
+
         table.append(
             [
                 index + 1,
-                ":",
                 worker_pool_summary.name,
-                f"[{worker_pool_summary.status}]",
-                f"[{worker_pool_summary.type.split('.')[-1:][0]}]",
+                f"{worker_pool_summary.status}",
+                # f"[{worker_pool_summary.type.split('.')[-1:][0]}]",
+                f"{nodes_running}",
+                f"{min_nodes}",
+                f"{max_nodes}",
+                f"{nodes_terminated}",
+                f"{nodes_late}",
+                f"{nodes_lost}",
+                f"{nodes_deregistered}",
             ]
         )
-    return table
+    return headers, table
 
 
 def print_numbered_object_list(
@@ -191,8 +248,9 @@ def print_numbered_object_list(
     )
     print()
 
-    if isinstance(objects[0], ComputeRequirementSummary):
-        table = compute_requirement_table(objects)
+    headers = None
+    if isinstance(objects[0], ComputeRequirement):
+        headers, table = compute_requirement_table(objects)
     elif isinstance(objects[0], WorkRequirementSummary):
         table = work_requirement_table(objects)
     elif isinstance(objects[0], TaskGroup):
@@ -200,13 +258,17 @@ def print_numbered_object_list(
     elif isinstance(objects[0], Task):
         table = task_table(objects)
     elif isinstance(objects[0], WorkerPoolSummary):
-        table = worker_pool_table(objects)
+        headers, table = worker_pool_table(client, objects)
     else:
         table = []
         for index, obj in enumerate(objects):
             table.append([index + 1, ":", obj.name])
-
-    print(indent(tabulate(table, tablefmt="plain"), indent_width=4))
+    if headers is None:
+        print(indent(tabulate(table, tablefmt="plain"), indent_width=4))
+    else:
+        print(
+            indent(tabulate(table, headers=headers, tablefmt="simple"), indent_width=4)
+        )
     print()
 
 
@@ -334,7 +396,15 @@ def print_compute_template_test_result(result: ComputeRequirementTemplateTestRes
     report: BestComputeSourceReport = result.report
     sources: List[BestComputeSourceReportSource] = report.sources
     source_table = [
-        ["Sequence", "Rank", "Provider", "Type", "Region", "InstanceType", "Source Name"]
+        [
+            "",
+            "Rank",
+            "Provider",
+            "Type",
+            "Region",
+            "InstanceType",
+            "Source Name",
+        ]
     ]
     for index, source in enumerate(sources):
         source_table.append(
@@ -349,5 +419,40 @@ def print_compute_template_test_result(result: ComputeRequirementTemplateTestRes
             ]
         )
     print()
-    print(tabulate(source_table, headers="firstrow", tablefmt="simple_grid"))
+    print(tabulate(source_table, headers="firstrow", tablefmt="simple_outline"))
+    print()
+
+
+def print_object_detail(object_detail: ObjectDetail):
+    """
+    Pretty print an Object Detail
+    """
+    indent: str = 4 * " "
+    print(f"{indent}Namespace:         {object_detail.namespace}")
+    print(f"{indent}Object Name:       {object_detail.objectName}")
+    print(f"{indent}Object Size:       {object_detail.objectSize:,d} byte(s)")
+    print(f"{indent}Last Modified At:  {object_detail.lastModified}")
+
+
+def print_batch_upload_files(upload_batch_builder: UploadBatchBuilder):
+    """
+    Print the list of files that will be batch uploaded
+    """
+    if ARGS_PARSER.quiet:
+        return
+
+    headers = ["Item", "Source Object", "->", "Target Object"]
+    table = []
+    # Yes, I know I shouldn't be accessing '_source_file_entries'
+    for index, file_entry in enumerate(upload_batch_builder._source_file_entries):
+        table.append(
+            [
+                index + 1,
+                file_entry.source_file_path,
+                "->",
+                f"{upload_batch_builder.namespace}::{file_entry.default_object_name}",
+            ]
+        )
+    print()
+    print(indent(tabulate(table, headers=headers, tablefmt="simple"), indent_width=4))
     print()
